%% Photometric stereo demo

clear

% Knobs
inpath  = 'cat.mat';                % Input images
outpath = 'cat.stl';                % Output path
nIters = 1000;

% Load data
% I = images   (m x n x d)
% L = lighting (3 x d) 
% M = mask     (m x n)
load(inpath);
[m, n, ~] = size(I);

% Compute unit-normals from images
%N = compute_normals(I, L);      % TODO(you): include your function

% Compute gradients from normals
%
% NOTE: There's no "-" in the definition of DFDY because row indexing of
%       matrices in MATLAB is reversed (row 1 = top) and so the sign must
%       be flipped in order for the y-gradients to be oriented correctly
%
DFDX = -N(:,:,1) ./ N(:,:,3);       
DFDY =  N(:,:,2) ./ N(:,:,3);
DFDX(isnan(DFDX) | ~M) = 0;         % Clean data and apply mask
DFDY(isnan(DFDY) | ~M) = 0;         % Clean data and apply mask

% Construct least-squares problem from gradients
%A = first_diffs_2d_matrix(m,n); % TODO(you): include your function
%b = ??                         % TODO(you): form b

% Built-in solver
[fxy,~] = lsqr(A,b,[],nIters);

% Format surface
FXY = reshape(fxy,[m,n]);         % Reshape into matrix
FXY = FXY - min(FXY(M));            % Anchor to z-axis
FXY = FXY .* M;                     % Apply mask

% Plot results
figure;
subplot(2,2,1); imshow(DFDX, []); title('x-gradients'); colorbar;
subplot(2,2,2); imshow(DFDY, []); title('y-gradients'); colorbar;
subplot(2,2,3); imshow(FXY, []);  title('depth map');   colorbar;
subplot(2,2,4); surfplot(FXY);    title('surface');

%% Write stl file
%[XX, YY] = meshgrid(1:n, m:-1:1);
%FV = surf2solid(XX, YY, FXY, 'elevation', 0);
%stlwrite(outpath, FV);
%fprintf('%s written\n', outpath);
